# ICE Project: "An Optimal Rule of Thumb for Pollution Permits Allocation"  
# by Mar Reguant and Eva Dardati
# August, 2008
#
# Questions or comments to mreguant@mit.edu
#
# References: Please, have a look at Sven Leyffer, Todd Munson and Karl Schmedders tutorials
# Model based on "Cournot Game with Learning and Investment" in K. Schmedders slides and
# "Incorporating oligopoly, CO2 emissions trading and green certificates into a
# power generation expansion model" by Pedro Linares et al, Automatica.

# Sets
	set firms ordered := 1 .. 2;		# firms --- code only prepared to have two!!!!!!!!
	param nStates = 20;
	set states := (1 .. nStates cross 1 .. nStates);	# combinations of states
	set transitions := -1 .. 1; # transitions in theta (3)
	
# Parameters
	# Demand parameters
	param A := 10;		# Intercept
	param b := 2;		# Slope

	# CO2 price equation
	param gamma := 1;	# Slope
	param q0 := 12;		# Emissions of small polluters at price CO2 = 0
	param zf := 0;		# Allocation given to small polluters
	param z{firms cross states}, default 4; # Allocation given to big polluters

	# Dynamic
	param d := 1;   	# Investment cost
	param delta := 0.9;	# Discount rate
	param theta{i in firms, (v, w) in states} := (v-1)/(nStates-1);

	
# Variables 
	# to solve for
	var q{firms cross states} >= 0;			# Firm quantity at each state - choice
	var x{firms cross states} >= 0;			# Firm investment at each state - choice
	var V{i in firms, v in 0 .. (nStates + 1), w in 0 .. (nStates + 1)}, default 1, >= 0;	# Value "function" at each state

	# auxiliary
	var Pi{i in firms, (v,w) in states} = 						# Static game profits		
		(A - b*q[nextw(i),w,v] - b*q[i,v,w])*q[i,v,w] - gamma*(theta[i,v,w]*q[i,v,w] - z[i,v,w] +
		theta[nextw(i),w,v]*q[nextw(i),w,v] - z[nextw(i),w,v] + q0 - zf)*(theta[i,v,w]*q[i,v,w] - z[i,v,w]) - d*x[i,v,w]^2;
		
	var Pr{i in firms, (v,w) in states, t in transitions} = 		# Transition probabilities
		(if t = -1 then theta[i,v,w]*x[i,v,w]/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)) +
		(if t = 0 then 1/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)) +
		(if t = 1 then (1-theta[i,v,w])/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)) ;
		
	var dPr{i in firms, (v,w) in states, t in transitions} = 	# Derivative of transition probabilities
		(if t = -1 then -(theta[i,v,w]-2)*theta[i,v,w]/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)^2) +
		(if t = 0 then  - theta[i,v,w]/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)^2) +
		(if t = 1 then  -(1-theta[i,v,w])*theta[i,v,w]/(theta[i,v,w]*x[i,v,w]-theta[i,v,w]+2)^2) ;
	
	
subject to
	Bellman{i in firms, (v,w) in states}:
	  V[i,v,w] = Pi[i,v,w] + delta*sum{t in transitions, k in transitions} (Pr[i,v,w,t]*Pr[nextw(i),w,v,k]*
	  V[i,v + t,w + k]) ;
  
	dVdq{i in firms, (v,w) in states}:
	  A - b*q[nextw(i),w,v] - 2*b*q[i,v,w] 
	  - gamma*theta[i,v,w]*(-z[i,v,w]+q[i,v,w]*theta[i,v,w]-z[nextw(i),w,v]+q[nextw(i),w,v]*theta[nextw(i),w,v]-zf+q0)-
	  gamma*theta[i,v,w]*(q[i,v,w]*theta[i,v,w]-z[i,v,w]) = 0;
	
# for path
	dVdx{i in firms, (v,w) in states}:
	  0 <= x[i,v,w] complements 
	  -2*d*x[i,v,w] + delta*sum{t in transitions, k in transitions} dPr[i,v,w,t]*Pr[nextw(i),w,v,k]*V[i,v + t,w + k] <= 0;
	

# Initialization	  
fix{i in firms, v in 0 .. nStates+1 by nStates+1, w in 0 .. nStates+1} V[i, v, w] := 0;
fix{i in firms, v in 0 .. nStates+1, w in 0 .. nStates+1 by nStates+1} V[i, v, w] := 0;
let{i in firms, (v, w) in states} q[i,v,w] := 2;
let{i in firms, (v, w) in states} x[i,v,w] := 0.25;